package cn.com.qmth.examcloud.task.api.provider;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.task.api.request.SyncCourseReq;
import cn.com.qmth.examcloud.task.api.request.SyncExamReq;
import cn.com.qmth.examcloud.task.api.request.SyncExamStudentReq;
import cn.com.qmth.examcloud.task.api.request.SyncOrgReq;
import cn.com.qmth.examcloud.task.api.request.SyncSpecialtyReq;
import cn.com.qmth.examcloud.task.api.request.SyncStudentReq;
import cn.com.qmth.examcloud.task.api.response.SyncCourseResp;
import cn.com.qmth.examcloud.task.api.response.SyncExamResp;
import cn.com.qmth.examcloud.task.api.response.SyncExamStudentResp;
import cn.com.qmth.examcloud.task.api.response.SyncOrgResp;
import cn.com.qmth.examcloud.task.api.response.SyncSpecialtyResp;
import cn.com.qmth.examcloud.task.api.response.SyncStudentResp;
import cn.com.qmth.examcloud.task.dao.DataSyncRepo;
import cn.com.qmth.examcloud.task.dao.entity.DataSyncEntity;

/**
 * 数据同步
 *
 * @author WANGWEI
 * @date 2018年8月2日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("${$rmp.cloud.task}" + "dataSync")
public class DataSyncCloudServiceProvider extends ControllerSupport
		implements
			DataSyncCloudService {

	@Autowired
	DataSyncRepo dataSyncRepo;

	@Autowired
	HandleSyncCloudService handleSyncCloudService;

	private static final long serialVersionUID = -2880611326177571371L;

	@PostMapping("syncCourse")
	@Override
	public SyncCourseResp syncCourse(@RequestBody SyncCourseReq req) {

		return null;
	}

	@PostMapping("syncOrg")
	@Override
	public SyncOrgResp syncOrg(@RequestBody SyncOrgReq req) {

		return null;
	}

	@PostMapping("syncStudent")
	@Override
	public SyncStudentResp syncStudent(@RequestBody SyncStudentReq req) {

		String group = PropertiesUtil.getString("$sync.syncStudent.group");
		if (StringUtils.isBlank(group)) {
			throw new StatusException("T-001001", "group is not configured");
		}

		List<String> componentList = RegExpUtil.findAll(group, "[^\\,]+");

		if (CollectionUtils.isEmpty(componentList)) {
			throw new StatusException("T-001001", "no component");
		}

		cn.com.qmth.examcloud.commons.api.request.SyncStudentReq request = new cn.com.qmth.examcloud.commons.api.request.SyncStudentReq();
		request.setEnable(req.getEnable());
		request.setId(req.getId());
		request.setIdentityNumber(req.getIdentityNumber());
		request.setName(req.getName());
		request.setOrgCode(req.getOrgCode());
		request.setOrgId(req.getOrgId());
		request.setOrgName(req.getOrgName());
		request.setPhoneNumber(req.getPhoneNumber());
		request.setPhotoPath(req.getPhotoPath());
		request.setRootOrgId(req.getRootOrgId());
		request.setSecurityPhone(req.getSecurityPhone());
		request.setStudentCode(req.getStudentCode());
		request.setSyncType(req.getSyncType());

		for (String component : componentList) {
			String url = PropertiesUtil.getString("$sync.syncStudent.component." + component);
			request.setUrl(url);
			try {
				handleSyncCloudService.syncStudent(request);
			} catch (Exception e) {
				DataSyncEntity entity = new DataSyncEntity();
				entity.setMethodName("syncCourse");
				entity.setParamType(
						cn.com.qmth.examcloud.commons.api.request.SyncStudentReq.class.getName());
				entity.setSyncNum(0);
				entity.setParamJson(JsonUtil.toJson(request));
				dataSyncRepo.saveAndFlush(entity);
			}
		}

		SyncStudentResp resp = new SyncStudentResp();
		return resp;
	}

	@PostMapping("syncSpecialty")
	@Override
	public SyncSpecialtyResp syncSpecialty(@RequestBody SyncSpecialtyReq req) {

		return null;
	}

	@PostMapping("syncExam")
	@Override
	public SyncExamResp syncExam(@RequestBody SyncExamReq req) {
		return null;
	}

	@PostMapping("syncExamStudent")
	@Override
	public SyncExamStudentResp syncExamStudent(@RequestBody SyncExamStudentReq req) {
		return null;
	}

}
